import random as _rnd
from random import Random as _Random
from string import ascii_uppercase as _ascii_uppercase, digits as _digits
import os as _os
import math as _math


class _Seeder():
    def __init__(self, seed):
        self.seeder = _Random(seed)
        self.seen = {}
        self.init_state = self.seeder.getstate()

    def get_seed(self, row, col):
        seed = None
        if ((col, row) in self.seen):
            state = self.seen[(col, row)]
            self.seeder.setstate(state)
        else:
            self.seeder.setstate(self.init_state)
        seed = ''.join(self.seeder.choices(_ascii_uppercase + _digits, k=128))
        self.seen[(col, row)] = self.seeder.getstate()
        return seed


# Set up the global seeder
_seeder = None
if ('BEE_SEARCH_SEED' in _os.environ):
    _seeder = _Seeder(_os.environ['BEE_SEARCH_SEED'])


class Trajectory:
    """
    A trajectory for a moving entity. The trajectory is a single straight-line
    movement of an entity and includes the direction the entity will move in,
    and the distance it will move.
    """
    def __init__(self, direction: float, distance: int, degrees=False):
        """Create a new trajectory with the given direction and distance. The
        direction can be specified in radians or degrees (default radians)"""
        self.distance = distance
        if (degrees):
            self.direction = _math.radians(direction)
        else:
            self.direction = direction

    def get_direction_in_degrees(self) -> float:
        """Retrieve the direction component of the trajectory in degrees"""
        return _math.degrees(self.direction)

    def get_direction_in_radians(self) -> float:
        """Retrieve the direction component of the trajectory in radians"""
        return self.direction

    def get_distance(self) -> int:
        """Retrieve the distance component of the trajectory"""
        return self.distance

    def __str__(self):
        return "("+"{:.2f}".format(self.direction)+", "+str(self.distance)+")"


class Compass:
    """
    Supplies a single map entity with trajectories which dictate the path the
    entity will take. NOTE: Each map entity MUST USE A SEPARATE compass.
    """
    def __init__(self, row: int, col: int, speed: int):
        """
        Create a new compass for the map entity starting at the given location,
        which gives directions in the form of trajectories to the entity based
        on the given speed of the entity.
        """
        self.speed = speed
        if (_seeder is not None):
            seed = _seeder.get_seed(row, col)
            self.rand = _Random(str(seed)+str(row)+str(col))
        else:
            self.rand = _Random()
        self.scout = bool(self.rand.getrandbits(1))

    def get_next_trajectory(self) -> Trajectory:
        """Return the next trajectory that the map entity must take."""
        # Generate direction
        angle = self.rand.vonmisesvariate(0, 0)
        direction = (angle//(0.25*_math.pi))*(0.25*_math.pi)
        # Generate distance
        distance = _math.ceil(self.rand.triangular(0, self.speed, self.speed))
        # return trajectory
        return Trajectory(direction, distance)

    def is_scout(self) -> bool:
        """
        When this compass is given to a Honey Bee, determine if the Honey Bee
        should be a scout or a forager. This method can be called multiple
        times and will always return the same result for a particular Honey Bee.
        """
        return self.scout



from setuptools import find_packages, setup
from distutils.core import setup
from setuptools.command.install import install
from os import chmod

stdlib_modules=['compass',
    'handin1api']

with open("README.md", "r") as fh:
    long_description = fh.read()

class MyInstall(install):
    def run(self):
        # Perform a normal install.
        install.run(self)
        # Change permissions of the installed .py and .pyc files.
        print(self.get_outputs())
        for fileName in self.get_outputs():
            if fileName.endswith(('.py', '.pyc')):
                for stdlib_module in stdlib_modules:
                    if stdlib_module in fileName:
                        chmod(fileName, 420)  # 420 decimal = 644 octal
                        break

setup(
    name="su-cs144-projectlibs",
    version="0.0.3",
    description='Stellenbosch University Python 3 code for 2024 CS144 semester project.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Marcel Dunaiski",
    author_email='marceldunaiski@sun.ac.za',
    #url="...",
    license="GNU General Public License v3 (GPLv3)",
    python_requires='>=3.8',
    packages=find_packages(exclude=["examples", "tests"]),
    include_package_data=True,
    install_requires=[
    ],
    extras_require={
        "dev": [
            "pytest",
            "pytest-cov",
            "coveralls",
        ],
    },
    py_modules=stdlib_modules,
    package_dir={'': 'su/cs1/'},
    cmdclass={'install': MyInstall},
    zip_safe=False,
    platforms="any",
    classifiers=[
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Natural Language :: English",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Topic :: Utilities",
        "Intended Audience :: Education",
        "Intended Audience :: Science/Research",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)
